@@ -131,54 +131,48 @@ print_packets_string (GString *str,
 
 static inline void
 print_bytes_string (GString *str,
-		    gulong   bytes)
+		    guint64  bytes)
 {
-  bytes *= 10;
-
-  if (bytes > 2 << 30)
+  if (bytes > 1 << 30)
     {
-      bytes = bytes / (2 << 30);
+      bytes = (bytes * 10) / (1 << 30);
 
-      g_string_append_printf (str, " (%ld.%ld Gb)", bytes / 10, bytes % 10);
+      g_string_append_printf (str, " (%lld.%lld Gb)", bytes / 10, bytes % 10);
     }
-  else if (bytes > 2 << 20)
+  else if (bytes > 1 << 20)
     {
-      bytes = bytes / (2 << 20);
+      bytes = (bytes * 10) / (1 << 20);
 
-      g_string_append_printf (str, " (%ld.%ld Mb)", bytes / 10, bytes % 10);
+      g_string_append_printf (str, " (%lld.%lld Mb)", bytes / 10, bytes % 10);
     }
-  else if (bytes > 2 << 10)
+  else if (bytes > 1 << 10)
     {
-      bytes = bytes / (2 << 10);
+      bytes = (bytes * 10) / (1 << 10);
 
-      g_string_append_printf (str, " (%ld.%ld Kb)", bytes / 10, bytes % 10);
+      g_string_append_printf (str, " (%lld.%lld Kb)", bytes / 10, bytes % 10);
     }
   else if (bytes >= 0)
     {
-      g_string_append_printf (str, " (%ld.%ld b)", bytes / 10, bytes % 10);
+      g_string_append_printf (str, " (%lld.%lld b)", bytes / 10, bytes % 10);
     }
 }
 
 static void
 netstatus_dialog_update_activity (NetstatusDialogData *data)
 {
-  GString *str;
-  long     in_packets = 0;
-  long     out_packets = 0;
-  long     in_bytes = 0;
-  long     out_bytes = 0;
+  NetstatusStats  stats = { 0, };
+  GString        *str;
 
-  netstatus_iface_get_packet_statistics (data->iface, &in_packets, &out_packets);
-  netstatus_iface_get_byte_statistics (data->iface, &in_bytes, &out_bytes);
+  netstatus_iface_get_statistics (data->iface, &stats);
 
   str = g_string_new (NULL);
 
-  print_packets_string (str, out_packets);
-  print_bytes_string (str, out_bytes);
+  print_packets_string (str, stats.out_packets);
+  print_bytes_string (str, stats.out_bytes);
   gtk_label_set_text (GTK_LABEL (data->sent), str->str);
 	
-  print_packets_string (str, in_packets);
-  print_bytes_string (str, in_bytes);
+  print_packets_string (str, stats.in_packets);
+  print_bytes_string (str, stats.in_bytes);
   gtk_label_set_text (GTK_LABEL (data->received), str->str);
   
   g_string_free (str, TRUE);
@@ -325,12 +319,18 @@ netstatus_dialog_iface_state_changed (NetstatusIface      *iface,
 				      NetstatusDialogData *data)
 {
   netstatus_dialog_update_state (data);
-  netstatus_dialog_update_activity (data);
-  
   netstatus_dialog_update_inet4_support (data);
   netstatus_dialog_update_device_support (data);
 }
 
+static void
+netstatus_dialog_iface_stats_changed (NetstatusIface      *iface,
+				      GParamSpec          *pspec,
+				      NetstatusDialogData *data)
+{
+  netstatus_dialog_update_activity (data);
+}
+
 static void
 netstatus_dialog_iface_signal_strength_changed (NetstatusIface      *iface,
 						GParamSpec          *pspec,
@@ -745,6 +745,12 @@ netstatus_dialog_new (NetstatusIface *iface)
 					data,
 					data->dialog);
 
+  netstatus_connect_signal_while_alive (data->iface,
+					"notify::stats",
+					G_CALLBACK (netstatus_dialog_iface_stats_changed),
+					data,
+					data->dialog);
+
   netstatus_connect_signal_while_alive (data->iface,
 					"notify::name",
 					G_CALLBACK (netstatus_dialog_iface_name_changed),
