@@ -31,18 +31,33 @@
 #include "netstatus-enums.h"
 #include "netstatus-fallback-pixbuf.h"
 
+typedef enum
+{
+  NETSTATUS_SIGNAL_0_24 = 0,
+  NETSTATUS_SIGNAL_25_49,
+  NETSTATUS_SIGNAL_50_74,
+  NETSTATUS_SIGNAL_75_100,
+  NETSTATUS_SIGNAL_LAST
+} NetstatusSignal;
+
 struct _NetstatusIconPrivate
 {
   GtkWidget      *image;
+  GtkWidget      *signal_image;
   GtkWidget      *error_dialog;
 
   NetstatusIface *iface;
   NetstatusState  state;
+  NetstatusSignal signal_strength;
 
   GtkIconTheme   *icon_theme;
   GdkPixbuf      *icons [NETSTATUS_STATE_LAST];
   GdkPixbuf      *scaled_icons [NETSTATUS_STATE_LAST];
 
+  GdkPixbuf      *signal_icons [NETSTATUS_SIGNAL_LAST];
+  GdkPixbuf      *rotated_signal_icons [NETSTATUS_SIGNAL_LAST];
+  GdkPixbuf      *scaled_signal_icons [NETSTATUS_SIGNAL_LAST];
+
   GtkOrientation  orientation;
   int             size;
 
@@ -50,8 +65,11 @@ struct _NetstatusIconPrivate
 
   gulong          state_changed_id;
   gulong          name_changed_id;
+  gulong          wireless_changed_id;
+  gulong          signal_changed_id;
 
   guint           tooltips_enabled : 1;
+  guint           show_signal : 1;
 };
 
 enum {
@@ -63,12 +81,12 @@ enum {
   PROP_0,
   PROP_IFACE,
   PROP_ORIENTATION,
-  PROP_TOOLTIPS_ENABLED
+  PROP_TOOLTIPS_ENABLED,
+  PROP_SHOW_SIGNAL
 };
 
 static void netstatus_icon_init_pixbufs (NetstatusIcon *icon);
 static void netstatus_icon_scale_icons  (NetstatusIcon *icon,
-					 int            width,
 					 int            height);
 
 static GObjectClass *parent_class;
@@ -90,7 +108,7 @@ netstatus_icon_get_default_pixbuf (NetstatusIcon *icon)
 				 (gpointer) &fallback_pixbuf);
 
       return fallback_pixbuf;
-    }
+   }
 
   return g_object_ref (fallback_pixbuf);
 }
@@ -109,14 +127,21 @@ netstatus_icon_theme_changed (NetstatusIcon *icon,
       g_object_unref (icon->priv->icons [i]);
       icon->priv->icons [i] = NULL;
     }
+  
+  for (i = 0; i < NETSTATUS_SIGNAL_LAST; i++)
+    {
+      g_object_unref (icon->priv->scaled_signal_icons [i]);
+      icon->priv->scaled_signal_icons [i] = NULL;
+
+      g_object_unref (icon->priv->signal_icons [i]);
+      icon->priv->signal_icons [i] = NULL;
+    }
 
   netstatus_icon_init_pixbufs (icon);
 
   if (icon->priv->size)
     {
-      netstatus_icon_scale_icons (icon,
-				  icon->priv->size,
-				  icon->priv->size);
+      netstatus_icon_scale_icons (icon, icon->priv->size);
     }
 }
 
@@ -192,6 +217,72 @@ netstatus_icon_init_pixbuf (NetstatusIcon  *icon,
     }
 }
 
+static GdkPixbuf *
+rotate_pixbuf (GdkPixbuf *pixbuf)
+{
+  GdkPixbuf *freeme;
+  GdkPixbuf *retval;
+  guint32   *dest;
+  guint32   *src;
+  int        width;
+  int        height;
+  int        x;
+  int        y;
+
+  freeme = NULL;
+  if (!gdk_pixbuf_get_has_alpha (pixbuf))
+    pixbuf = freeme = gdk_pixbuf_add_alpha (pixbuf, FALSE, 0, 0, 0);
+
+  width  = gdk_pixbuf_get_width  (pixbuf);
+  height = gdk_pixbuf_get_height (pixbuf);
+
+  retval = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, height, width);
+
+  dest = (guint32 *) gdk_pixbuf_get_pixels (retval);
+  src  = (guint32 *) gdk_pixbuf_get_pixels (pixbuf);
+
+  for (y = 0; y < height; y++)
+    for (x = 0; x < width; x++)
+      dest [(height - y - 1) + height * (width - x - 1)] = src [y * width + x];
+
+  if (freeme)
+    g_object_unref (freeme);
+
+  return retval;
+}
+
+static void
+netstatus_icon_rotate_signal_icons (NetstatusIcon  *icon,
+				    GtkOrientation  orientation)
+{
+  int i;
+
+  if (!icon->priv->signal_icons [0])
+    return;
+
+  for (i = 0; i < NETSTATUS_SIGNAL_LAST; i++)
+    {
+      GdkPixbuf *pixbuf;
+
+      if (orientation == GTK_ORIENTATION_HORIZONTAL)
+	{
+	  pixbuf = g_object_ref (icon->priv->signal_icons [i]);
+	}
+      else /* if (orientation == GTK_ORIENTATION_VERTICAL) */
+	{
+	  pixbuf = rotate_pixbuf (icon->priv->signal_icons [i]);
+	}
+
+      if (icon->priv->rotated_signal_icons [i])
+	g_object_unref (icon->priv->rotated_signal_icons [i]);
+      icon->priv->rotated_signal_icons [i] = pixbuf;
+
+      if (icon->priv->scaled_signal_icons [i])
+	g_object_unref (icon->priv->scaled_signal_icons [i]);
+      icon->priv->scaled_signal_icons [i] = NULL;
+    }
+}
+
 static void
 netstatus_icon_init_pixbufs (NetstatusIcon *icon)
 {
@@ -224,6 +315,32 @@ netstatus_icon_init_pixbufs (NetstatusIcon *icon)
 			      &icon->priv->icons [NETSTATUS_STATE_ERROR],
 			      "gnome-netstatus-error");
   icon->priv->scaled_icons [NETSTATUS_STATE_ERROR] = NULL;
+
+  netstatus_icon_init_pixbuf (icon,
+			      &icon->priv->signal_icons [NETSTATUS_SIGNAL_0_24],
+			      "gnome-netstatus-0-24");
+  icon->priv->rotated_signal_icons [NETSTATUS_SIGNAL_0_24] = NULL;
+  icon->priv->scaled_signal_icons  [NETSTATUS_SIGNAL_0_24] = NULL;
+
+  netstatus_icon_init_pixbuf (icon,
+			      &icon->priv->signal_icons [NETSTATUS_SIGNAL_25_49],
+			      "gnome-netstatus-25-49");
+  icon->priv->rotated_signal_icons [NETSTATUS_SIGNAL_25_49] = NULL;
+  icon->priv->scaled_signal_icons  [NETSTATUS_SIGNAL_25_49] = NULL;
+
+  netstatus_icon_init_pixbuf (icon,
+			      &icon->priv->signal_icons [NETSTATUS_SIGNAL_50_74],
+			      "gnome-netstatus-50-74");
+  icon->priv->rotated_signal_icons [NETSTATUS_SIGNAL_50_74] = NULL;
+  icon->priv->scaled_signal_icons  [NETSTATUS_SIGNAL_50_74] = NULL;
+
+  netstatus_icon_init_pixbuf (icon,
+			      &icon->priv->signal_icons [NETSTATUS_SIGNAL_75_100],
+			      "gnome-netstatus-75-100");
+  icon->priv->rotated_signal_icons [NETSTATUS_SIGNAL_75_100] = NULL;
+  icon->priv->scaled_signal_icons  [NETSTATUS_SIGNAL_75_100] = NULL;
+
+  netstatus_icon_rotate_signal_icons (icon, icon->priv->orientation);
 }
 
 static void
@@ -238,7 +355,15 @@ netstatus_icon_update_image (NetstatusIcon *icon)
   if (!pixbuf)
     pixbuf = icon->priv->icons [icon->priv->state];
 
-  gtk_image_set_from_pixbuf (GTK_IMAGE (icon->priv->image), pixbuf);
+  if (gtk_image_get_pixbuf (GTK_IMAGE (icon->priv->image)) != pixbuf)
+    gtk_image_set_from_pixbuf (GTK_IMAGE (icon->priv->image), pixbuf);
+
+  pixbuf = icon->priv->scaled_signal_icons [icon->priv->signal_strength];
+  if (!pixbuf)
+    pixbuf = icon->priv->rotated_signal_icons [icon->priv->signal_strength];
+  
+  if (gtk_image_get_pixbuf (GTK_IMAGE (icon->priv->signal_image)) != pixbuf)
+    gtk_image_set_from_pixbuf (GTK_IMAGE (icon->priv->signal_image), pixbuf);
 }
 
 static void
@@ -285,6 +410,44 @@ netstatus_icon_state_changed (NetstatusIface *iface,
     }
 }
 
+static void
+netstatus_icon_is_wireless_changed (NetstatusIface *iface,
+				    GParamSpec     *pspec,
+				    NetstatusIcon  *icon)
+{
+  if (netstatus_iface_get_is_wireless (iface) && icon->priv->show_signal)
+    gtk_widget_show (icon->priv->signal_image);
+  else
+    gtk_widget_hide (icon->priv->signal_image);
+}
+
+static void
+netstatus_icon_signal_changed (NetstatusIface *iface,
+			       GParamSpec     *pspec,
+			       NetstatusIcon  *icon)
+{
+  NetstatusSignal signal_strength;
+  int             percent;
+
+  percent = netstatus_iface_get_signal_strength (iface);
+
+  if (percent < 25)
+    signal_strength = NETSTATUS_SIGNAL_0_24;
+  else if (percent < 50)
+    signal_strength = NETSTATUS_SIGNAL_25_49;
+  else if (percent < 75)
+    signal_strength = NETSTATUS_SIGNAL_50_74;
+  else
+    signal_strength = NETSTATUS_SIGNAL_75_100;
+
+  if (icon->priv->signal_strength != signal_strength)
+    {
+      icon->priv->signal_strength = signal_strength;
+
+      netstatus_icon_update_image (icon);
+    }
+}
+
 static void
 netstatus_icon_destroy (GtkObject *widget)
 {
@@ -302,9 +465,15 @@ netstatus_icon_destroy (GtkObject *widget)
 				   icon->priv->state_changed_id);
       g_signal_handler_disconnect (icon->priv->iface,
 				   icon->priv->name_changed_id);
+      g_signal_handler_disconnect (icon->priv->iface,
+				   icon->priv->wireless_changed_id);
+      g_signal_handler_disconnect (icon->priv->iface,
+				   icon->priv->signal_changed_id);
     }
-  icon->priv->state_changed_id = 0;
-  icon->priv->name_changed_id = 0;
+  icon->priv->state_changed_id    = 0;
+  icon->priv->name_changed_id     = 0;
+  icon->priv->wireless_changed_id = 0;
+  icon->priv->signal_changed_id   = 0;
 
   if (icon->priv->tooltips)
     g_object_unref (icon->priv->tooltips);
@@ -315,59 +484,129 @@ netstatus_icon_destroy (GtkObject *widget)
   GTK_OBJECT_CLASS (parent_class)->destroy (widget);
 }
 
+static GdkPixbuf *
+scale_pixbuf (GdkPixbuf      *pixbuf,
+	      GtkOrientation  orientation,
+	      int             size,
+	      gboolean        retain_aspect)
+{
+  GdkPixbuf *retval;
+  int        orig_size;
+
+  if (orientation == GTK_ORIENTATION_HORIZONTAL)
+    {
+      orig_size = gdk_pixbuf_get_height (pixbuf);
+    }
+  else /* if (orientation == GTK_ORIENTATION_VERTICAL) */
+    {
+      orig_size = gdk_pixbuf_get_width (pixbuf);
+    }
+
+  if (size >= orig_size)
+    {
+      retval = g_object_ref (pixbuf);
+    }
+  else
+    {
+      int    orig_width;
+      int    orig_height;
+      int    scaled_width;
+      int    scaled_height;
+      double scale;
+
+      orig_width  = gdk_pixbuf_get_width  (pixbuf);
+      orig_height = gdk_pixbuf_get_height (pixbuf);
+
+      scale = (double) size / orig_size;
+
+      if (retain_aspect)
+	{
+	  scaled_width  = orig_width  * scale;
+	  scaled_height = orig_height * scale;
+	}
+      else
+	{
+	  if (orientation == GTK_ORIENTATION_HORIZONTAL)
+	    {
+	      scaled_width  = orig_width;
+	      scaled_height = orig_height * scale;
+	    }
+	  else /* if (orientation == GTK_ORIENTATION_VERTICAL) */
+	    {
+	      scaled_width  = orig_width * scale;
+	      scaled_height = orig_height;
+	    }
+	}
+
+      retval = gdk_pixbuf_scale_simple (pixbuf,
+					scaled_width,
+					scaled_height,
+					GDK_INTERP_BILINEAR);
+    }
+
+  return retval;
+}
+
 static void
-netstatus_icon_scale_icons (NetstatusIcon *icon,
-			    int            width,
-			    int            height)
+netstatus_icon_scale_icons (NetstatusIcon  *icon,
+			    int             size)
 {
   int i;
 
-  g_return_if_fail (width  > 0);
-  g_return_if_fail (height > 0);
+  g_return_if_fail (size > 0);
 
   if (!icon->priv->icons [0])
     netstatus_icon_init_pixbufs (icon);
 
   for (i = 0; i < NETSTATUS_STATE_LAST; i++)
     {
-      int orig_width;
-      int orig_height;
-
       if (icon->priv->scaled_icons [i])
 	g_object_unref (icon->priv->scaled_icons [i]);
-      icon->priv->scaled_icons [i] = NULL;
+      icon->priv->scaled_icons [i] = scale_pixbuf (icon->priv->icons [i],
+						   icon->priv->orientation,
+						   size,
+						   TRUE);
+    }
 
-      orig_width  = gdk_pixbuf_get_width  (icon->priv->icons [i]);
-      orig_height = gdk_pixbuf_get_height (icon->priv->icons [i]);
-
-      if (width >= orig_width && height >= orig_height)
-	{
-	  icon->priv->scaled_icons [i] =
-	    g_object_ref (icon->priv->icons [i]);
-	}
-      else
-	{
-	  double scale;
-
-	  scale = MIN (((double) width)  / orig_width,
-		       ((double) height) / orig_height);
-	  
-	  icon->priv->scaled_icons [i] =
-	    gdk_pixbuf_scale_simple (icon->priv->icons [i],
-				     orig_width  * scale,
-				     orig_height * scale,
-				     GDK_INTERP_BILINEAR);
-	}
+  for (i = 0; i < NETSTATUS_SIGNAL_LAST; i++)
+    {
+      if (icon->priv->scaled_signal_icons [i])
+	g_object_unref (icon->priv->scaled_signal_icons [i]);
+      icon->priv->scaled_signal_icons [i] = scale_pixbuf (icon->priv->rotated_signal_icons [i],
+							  icon->priv->orientation,
+							  size,
+							  FALSE);
     }
 
   netstatus_icon_update_image (icon);
 }
 
+static inline GObjectClass *
+get_box_class (GtkOrientation orientation)
+{
+  return gtk_type_class (orientation == GTK_ORIENTATION_HORIZONTAL ? GTK_TYPE_HBOX : GTK_TYPE_VBOX);
+}
+
+static void
+netstatus_icon_size_request (GtkWidget      *widget,
+			     GtkRequisition *requisition)
+{
+  NetstatusIcon *icon = NETSTATUS_ICON (widget);
+  GObjectClass  *klass;
+
+  klass = get_box_class (icon->priv->orientation);
+
+  if (GTK_WIDGET_CLASS (klass)->size_request)
+    GTK_WIDGET_CLASS (klass)->size_request (widget, requisition);
+}
+
 static void
 netstatus_icon_size_allocate (GtkWidget     *widget,
 			      GtkAllocation *allocation)
 {
   NetstatusIcon *icon = (NetstatusIcon *) widget;
+  GtkAllocation  child_allocation;
+  GObjectClass  *klass;
   int            size;
 
   if (icon->priv->orientation == GTK_ORIENTATION_HORIZONTAL)
@@ -379,12 +618,66 @@ netstatus_icon_size_allocate (GtkWidget     *widget,
     {
       icon->priv->size = size;
 
-      netstatus_icon_scale_icons (icon, size, size);
+      netstatus_icon_scale_icons (icon, size);
     }
 
-  GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
+  if (GTK_WIDGET_REALIZED (widget))
+    {
+      gdk_window_move_resize (widget->window,
+                              allocation->x + GTK_CONTAINER (widget)->border_width,
+                              allocation->y + GTK_CONTAINER (widget)->border_width,
+                              MAX (allocation->width - GTK_CONTAINER (widget)->border_width * 2, 0),
+			      MAX (allocation->height - GTK_CONTAINER (widget)->border_width * 2, 0));
+    }
+
+  klass = get_box_class (icon->priv->orientation);
+
+  child_allocation.x = 0;
+  child_allocation.y = 0;
+  child_allocation.width  = MAX (allocation->width  - GTK_CONTAINER (widget)->border_width * 2, 0);
+  child_allocation.height = MAX (allocation->height - GTK_CONTAINER (widget)->border_width * 2, 0);
+
+  if (GTK_WIDGET_CLASS (klass)->size_allocate)
+    GTK_WIDGET_CLASS (klass)->size_allocate (widget, &child_allocation);
+
+  widget->allocation = *allocation;
 }
 
+static void
+netstatus_icon_realize (GtkWidget *widget)
+{
+  GdkWindowAttr attributes;
+  int           attributes_mask;
+  int           border_width;
+
+  GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
+  
+  border_width = GTK_CONTAINER (widget)->border_width;
+
+  attributes.x = widget->allocation.x + border_width;
+  attributes.y = widget->allocation.y + border_width;
+  attributes.width = widget->allocation.width - 2 * border_width;
+  attributes.height = widget->allocation.height - 2 * border_width;
+  attributes.window_type = GDK_WINDOW_CHILD;
+  attributes.wclass = GDK_INPUT_OUTPUT;
+  attributes.visual = gtk_widget_get_visual (widget);
+  attributes.colormap = gtk_widget_get_colormap (widget);
+  attributes.event_mask = gtk_widget_get_events (widget) |
+                          GDK_BUTTON_MOTION_MASK         |
+                          GDK_BUTTON_PRESS_MASK          |
+                          GDK_BUTTON_RELEASE_MASK        |
+                          GDK_EXPOSURE_MASK              |
+                          GDK_ENTER_NOTIFY_MASK          |
+                          GDK_LEAVE_NOTIFY_MASK;
+
+  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
+
+  widget->window = gdk_window_new (gtk_widget_get_parent_window (widget), &attributes, attributes_mask);
+  gdk_window_set_user_data (widget->window, widget);
+
+  widget->style = gtk_style_attach (widget->style, widget->window);
+  gtk_style_set_background (widget->style, widget->window, GTK_STATE_NORMAL);
+}
 
 static gboolean
 netstatus_icon_button_press_event (GtkWidget      *widget,
@@ -399,7 +692,6 @@ netstatus_icon_button_press_event (GtkWidget      *widget,
   return FALSE;
 }
 
-
 static void
 netstatus_icon_set_property (GObject      *object,
 			     guint         prop_id,
@@ -423,6 +715,9 @@ netstatus_icon_set_property (GObject      *object,
     case PROP_TOOLTIPS_ENABLED:
       netstatus_icon_set_tooltips_enabled (icon, g_value_get_boolean (value));
       break;
+    case PROP_SHOW_SIGNAL:
+      netstatus_icon_set_show_signal (icon, g_value_get_boolean (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -452,6 +747,9 @@ netstatus_icon_get_property (GObject    *object,
     case PROP_TOOLTIPS_ENABLED:
       g_value_set_boolean (value, icon->priv->tooltips_enabled);
       break;
+    case PROP_SHOW_SIGNAL:
+      g_value_set_boolean (value, icon->priv->show_signal);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -500,7 +798,9 @@ netstatus_icon_class_init (NetstatusIconClass *klass)
 
   gtkobject_class->destroy = netstatus_icon_destroy;
 
+  widget_class->size_request       = netstatus_icon_size_request;
   widget_class->size_allocate      = netstatus_icon_size_allocate;
+  widget_class->realize            = netstatus_icon_realize;
   widget_class->button_press_event = netstatus_icon_button_press_event;
 
   g_object_class_install_property (gobject_class,
@@ -528,6 +828,14 @@ netstatus_icon_class_init (NetstatusIconClass *klass)
 							 TRUE,
 							 G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
+  g_object_class_install_property (gobject_class,
+                                   PROP_SHOW_SIGNAL,
+                                   g_param_spec_boolean ("show-signal",
+							 _("Show Signal"),
+							 _("Whether or not the signal strength should be displayed."),
+							 TRUE,
+							 G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
   icon_signals [INVOKED] =
     g_signal_new ("invoked",
 		  G_OBJECT_CLASS_TYPE (gobject_class),
@@ -544,16 +852,24 @@ netstatus_icon_instance_init (NetstatusIcon      *icon,
 {
   icon->priv = g_new0 (NetstatusIconPrivate, 1);
 
+  GTK_WIDGET_UNSET_FLAGS (icon, GTK_NO_WINDOW);
+
   icon->priv->iface            = NULL;
   icon->priv->state            = NETSTATUS_STATE_DISCONNECTED;
   icon->priv->orientation      = GTK_ORIENTATION_HORIZONTAL;
   icon->priv->size             = 0;
   icon->priv->state_changed_id = 0;
 
+  gtk_box_set_spacing (GTK_BOX (icon), 3);
+
   icon->priv->image = gtk_image_new ();
   gtk_container_add (GTK_CONTAINER (icon), icon->priv->image);
   gtk_widget_show (icon->priv->image);
 
+  icon->priv->signal_image = gtk_image_new ();
+  gtk_container_add (GTK_CONTAINER (icon), icon->priv->signal_image);
+  gtk_widget_hide (icon->priv->signal_image);
+
   icon->priv->tooltips = gtk_tooltips_new ();
   g_object_ref (icon->priv->tooltips);
   gtk_object_sink (GTK_OBJECT (icon->priv->tooltips));
@@ -583,7 +899,7 @@ netstatus_icon_get_type (void)
 	  NULL
 	};
 
-      type = g_type_register_static (GTK_TYPE_EVENT_BOX, "NetstatusIcon", &info, 0);
+      type = g_type_register_static (GTK_TYPE_BOX, "NetstatusIcon", &info, 0);
     }
 
   return type;
@@ -666,6 +982,10 @@ netstatus_icon_set_iface (NetstatusIcon  *icon,
 				       icon->priv->state_changed_id);
 	  g_signal_handler_disconnect (icon->priv->iface,
 				       icon->priv->name_changed_id);
+	  g_signal_handler_disconnect (icon->priv->iface,
+				       icon->priv->wireless_changed_id);
+	  g_signal_handler_disconnect (icon->priv->iface,
+				       icon->priv->signal_changed_id);
 	}
 
       if (iface)
@@ -675,13 +995,19 @@ netstatus_icon_set_iface (NetstatusIcon  *icon,
       if (old_iface)
 	g_object_unref (old_iface);
 
-      icon->priv->state_changed_id = g_signal_connect (icon->priv->iface, "notify::state",
-						       G_CALLBACK (netstatus_icon_state_changed), icon);
-      icon->priv->name_changed_id  = g_signal_connect (icon->priv->iface, "notify::name",
-						       G_CALLBACK (netstatus_icon_name_changed), icon);
+      icon->priv->state_changed_id     = g_signal_connect (icon->priv->iface, "notify::state",
+							   G_CALLBACK (netstatus_icon_state_changed), icon);
+      icon->priv->name_changed_id      = g_signal_connect (icon->priv->iface, "notify::name",
+							   G_CALLBACK (netstatus_icon_name_changed), icon);
+      icon->priv->wireless_changed_id  = g_signal_connect (icon->priv->iface, "notify::wireless",
+							   G_CALLBACK (netstatus_icon_is_wireless_changed), icon);
+      icon->priv->signal_changed_id    = g_signal_connect (icon->priv->iface, "notify::signal-strength",
+							   G_CALLBACK (netstatus_icon_signal_changed), icon);
 
-      netstatus_icon_state_changed (icon->priv->iface, NULL, icon);
-      netstatus_icon_name_changed  (icon->priv->iface, NULL, icon);
+      netstatus_icon_state_changed       (icon->priv->iface, NULL, icon);
+      netstatus_icon_name_changed        (icon->priv->iface, NULL, icon);
+      netstatus_icon_is_wireless_changed (icon->priv->iface, NULL, icon);
+      netstatus_icon_signal_changed      (icon->priv->iface, NULL, icon);
 
       g_object_notify (G_OBJECT (icon), "iface");
     }
@@ -705,6 +1031,13 @@ netstatus_icon_set_orientation (NetstatusIcon  *icon,
     {
       icon->priv->orientation = orientation;
 
+      netstatus_icon_rotate_signal_icons (icon, orientation);
+      netstatus_icon_update_image (icon);
+
+      icon->priv->size = -1;
+
+      gtk_widget_queue_resize (GTK_WIDGET (icon));
+
       g_object_notify (G_OBJECT (icon), "orientation");
     }
 }
@@ -745,3 +1078,32 @@ netstatus_icon_get_tooltips_enabled (NetstatusIcon *icon)
 
   return icon->priv->tooltips_enabled;
 }
+
+void
+netstatus_icon_set_show_signal (NetstatusIcon *icon,
+				gboolean       show_signal)
+{
+  g_return_if_fail (NETSTATUS_IS_ICON (icon));
+
+  show_signal = show_signal != FALSE;
+
+  if (icon->priv->show_signal != show_signal)
+    {
+      icon->priv->show_signal = show_signal;
+
+      if (show_signal && netstatus_iface_get_is_wireless (icon->priv->iface))
+	gtk_widget_show (icon->priv->signal_image);
+      else
+	gtk_widget_hide (icon->priv->signal_image);
+
+      g_object_notify (G_OBJECT (icon), "show-signal");
+    }
+}
+
+gboolean
+netstatus_icon_get_show_signal (NetstatusIcon *icon)
+{
+  g_return_val_if_fail (NETSTATUS_ICON (icon), TRUE);
+
+  return icon->priv->show_signal;
+}
